package ast;

import java.util.List;

import types.Type;

public class Menor extends Node implements Exp {
	public Exp l;
	public Exp r;
	
	public Menor(Exp l, Exp r, int line, int col) {
		super(line, col);
		this.l = l;
		this.r = r;
	}

	@Override
	public Object eval(Environment<Object> env) {
		Comparable<Object> c = (Comparable<Object>)l.eval(env);
		return c.compareTo(r.eval(env)) < 0;
	}

	@Override
	public Type tcExp(Environment<Type> tenv) {
		Type tl = l.tcExp(tenv);
		if(tl == types.Num.type || tl == types.Str.type) {
			Type tr = r.tcExp(tenv);
			tl.checkEq(tr, this);
			return types.Bool.type;
		} else
			typeError(tl, types.Num.type);
		return null;
	}

	@Override
	public String label() {
		return "<";
	}

	@Override
	public Exp ccExp(Environment<LocId> env, PrimFunc cur, List<PrimFunc> funcs) {
		return new Menor(l.ccExp(env, cur, funcs),
				r.ccExp(env, cur, funcs), line, col);
	}

}
